SystemOrganization addCategory: #Brltty!
SystemOrganization addCategory: #'Brltty-Tests'!

TestCase subclass: #BrlttyTests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Brltty-Tests'!

!BrlttyTests methodsFor: 'tests' stamp: 'ThierryGoubier 4/3/2012 11:12'!
testConnectToBrltty
	"Test the connection to brltty."
	
	| s |
	s := Socket new.
	s connectTo: NetNameResolver localHostAddress port: 4101.
	s waitForConnectionFor: Socket standardTimeout.
	s isConnected ifFalse: [Socket error: 's not connected'].
	s close.
	s isUnconnectedOrInvalid ifFalse: [Socket error: 's not closed'].
	s destroy.
	! !

!BrlttyTests methodsFor: 'tests' stamp: 'ThierryGoubier 4/3/2012 13:17'!
testReadBrlttyVersion
	"Test reading the version of brltty once connected."
	
	| s d sz type version |
	s := Socket new.
	BrlttyConnection current: s.
	s connectTo: NetNameResolver localHostAddress port: 4101.
	s waitForConnectionFor: Socket standardTimeout.
	s isConnected ifFalse: [Socket error: 's not connected'].
	d := s receiveData asByteArray.
	sz := Integer byte1: (d at: 4) byte2: (d at: 3) byte3: (d at: 2) byte4: (d at: 1).
	type := (d at: 8) asCharacter.
	version := Integer byte1: (d at: 12) byte2: (d at: 11) byte3: (d at: 10) byte4: (d at: 9).
	Transcript show: ('Brltty connect replies: (size = ' , sz printString ,  ', type = ' , type printString , ', version = ' , version printString, ')'); cr.
	s close.
	s isUnconnectedOrInvalid ifFalse: [Socket error: 's not closed'].
	s destroy.
	! !

!BrlttyTests methodsFor: 'tests' stamp: 'ThierryGoubier 4/3/2012 16:03'!
testReadBrlttyVersionAndAuth
	"Test reading the version and the auth of brltty once connected."

	| s d sz type version |
	s := Socket new.
	BrlttyConnection current: s.
	s connectTo: NetNameResolver localHostAddress port: 4101.
	s waitForConnectionFor: Socket standardTimeout.
	s isConnected
		ifFalse: [ Socket error: 's not connected' ].
	d := s receiveData asByteArray.
	sz := Integer
		byte1: (d at: 4)
		byte2: (d at: 3)
		byte3: (d at: 2)
		byte4: (d at: 1).
	type := (d at: 8) asCharacter.
	version := Integer
		byte1: (d at: 12)
		byte2: (d at: 11)
		byte3: (d at: 10)
		byte4: (d at: 9).
	Transcript
		show:
				'Brltty connect replies: (size = ' , sz printString , ', type = ' , type printString , ', version = '
						, version printString , ')';
		cr.
	s sendData: d.
	sz := 4.
	d := ByteArray new.
	[ d size < (sz + 8) ]
		whileTrue: [ 
			d := d , s receiveData asByteArray.
			sz := Integer
				byte1: (d at: 4)
				byte2: (d at: 3)
				byte3: (d at: 2)
				byte4: (d at: 1) ].
	type := (d at: 8) asCharacter.
	version := (d at: 12) asCharacter.
	Transcript
		show:
				'Brltty connect replies: (size = ' , sz printString , ', type = ' , type printString , ', authorisation = '
						, version printString , ')';
		cr.
	s close.
	s isUnconnectedOrInvalid
		ifFalse: [ Socket error: 's not closed' ].
	s destroy! !

Object subclass: #BrlttyConnection
	instanceVariableNames: 'socket stream'
	classVariableNames: 'current'
	poolDictionaries: ''
	category: 'Brltty'!

!BrlttyConnection class methodsFor: 'accessing' stamp: 'ThierryGoubier 4/3/2012 11:03'!
current
	^ current! !

!BrlttyConnection class methodsFor: 'accessing' stamp: 'ThierryGoubier 4/3/2012 11:03'!
current: anObject
	current := anObject! !

!BrlttyConnection class methodsFor: 'comments' stamp: 'ThierryGoubier 4/3/2012 15:50'!
packetFormat
	"Here is the format for handling communication to the brltty server.

a packet format is :
uint32 size;
uint32 type;
char[size] data;

#define BRLAPI_MAXPACKETSIZE 512

#define BRLAPI_PACKET_VERSION         'v'   /**< Version                     */
#define BRLAPI_PACKET_AUTH            'a'   /**< Authorization               */
#define BRLAPI_PACKET_GETDRIVERNAME   'n'   /**< Ask which driver is used    */
#define BRLAPI_PACKET_GETDISPLAYSIZE  's'   /**< Dimensions of brl display   */
#define BRLAPI_PACKET_ENTERTTYMODE    't'   /**< Asks for a specified tty    */
#define BRLAPI_PACKET_SETFOCUS        'F'   /**< Set current tty focus       */
#define BRLAPI_PACKET_LEAVETTYMODE    'L'   /**< Release the tty             */
#define BRLAPI_PACKET_KEY             'k'   /**< Braille key                 */
#define BRLAPI_PACKET_IGNOREKEYRANGES 'm'   /**< Mask key ranges             */
#define BRLAPI_PACKET_ACCEPTKEYRANGES 'u'   /**< Unmask key ranges           */
#define BRLAPI_PACKET_WRITE           'w'   /**< Write                       */
#define BRLAPI_PACKET_ENTERRAWMODE    '*'   /**< Enter in raw mode           */
#define BRLAPI_PACKET_LEAVERAWMODE    '#'   /**< Leave raw mode              */
#define BRLAPI_PACKET_PACKET          'p'   /**< Raw packets                 */
#define BRLAPI_PACKET_ACK             'A'   /**< Acknowledgement             */
#define BRLAPI_PACKET_ERROR           'e'   /**< non-fatal error             */
#define BRLAPI_PACKET_EXCEPTION       'E'   /**< Exception                   */
#define BRLAPI_PACKET_SUSPENDDRIVER   'S'   /**< Suspend driver              */
#define BRLAPI_PACKET_RESUMEDRIVER    'R'   /**< Resume driver               */
"! !

!BrlttyConnection methodsFor: 'accessing' stamp: 'ThierryGoubier 4/3/2012 15:51'!
socket
	^ socket! !

!BrlttyConnection methodsFor: 'accessing' stamp: 'ThierryGoubier 4/3/2012 15:51'!
socket: anObject
	socket := anObject! !

!BrlttyConnection methodsFor: 'accessing' stamp: 'ThierryGoubier 4/3/2012 15:51'!
stream
	^ stream! !

!BrlttyConnection methodsFor: 'accessing' stamp: 'ThierryGoubier 4/3/2012 15:51'!
stream: anObject
	stream := anObject! !
